@@ -8,10 +8,11 @@ package netns
 import (
 	"syscall"
 
+	"tailscale.com/net/netmon"
 	"tailscale.com/types/logger"
 )
 
-func control(logger.Logf) func(network, address string, c syscall.RawConn) error {
+func control(logger.Logf, *netmon.Monitor) func(network, address string, c syscall.RawConn) error {
 	return controlC
 }
 
